@@ -7,11 +7,11 @@
 setClass("lavData",
     representation(
         data.type="character",     # "full", "moment" or "none"
-        ngroups="integer",         # number of groups
         group="character",         # group variable
-        nlevels="integer",         # number of levels
-        cluster="character",       # cluster variable(s)
+        ngroups="integer",         # number of groups
         group.label="character",   # group labels
+        cluster="character",       # cluster variable(s)
+        nlevels="integer",         # number of levels
         level.label="character",   # level labels
         std.ov="logical",          # standardize observed variables?
         nobs="list",               # effective number of observations
@@ -102,6 +102,7 @@ setClass("lavModel",          # MATRIX representation of the sem model
 
         nblocks="integer",
         ngroups="integer",   # only for rsem!! (which uses rsem:::computeDelta)
+        nefa="integer",
         nmat="integer",
         nvar="integer",
         num.idx="list",
@@ -139,6 +140,7 @@ setClass("lavModel",          # MATRIX representation of the sem model
         cin.rhs="numeric",
         cin.linear.idx="integer",
         cin.nonlinear.idx="integer",
+        ceq.efa.JAC="matrix",
         con.jac="matrix",
         con.lambda="numeric",
 
@@ -151,6 +153,11 @@ setClass("lavModel",          # MATRIX representation of the sem model
         ov.y.dummy.ov.idx="list",
         ov.y.dummy.lv.idx="list",
 
+        ov.efa.idx="list",
+        lv.efa.idx="list",
+        H="list",
+        lv.order="list",
+
         estimator="character"
     )
 )
@@ -225,6 +232,7 @@ setClass("lavaanList",
         impliedList     = "list",
         h1List          = "list",
         loglikList      = "list",
+        baselineList    = "list",
         funList         = "list",
         external        = "list"       # optional slot, for add-on packages
     )
