@@ -1,24 +1,30 @@
-# user-visible routine to 
+# user-visible routine to
 # compute polychoric/polyserial/... correlations
 #
 # YR 17 Sept 2013
-# 
+#
 # - YR 26 Nov 2013: big change - make it a wrapper around lavaan()
 #                   estimator = "none" means two.step (starting values)
 
-lavCor <- function(object, 
+lavCor <- function(object,
                    # lav.data options
-                   ordered    = NULL, 
-                   group      = NULL, 
+                   ordered    = NULL,
+                   group      = NULL,
                    missing    = "listwise",
-                   ov.names.x = NULL, 
+                   ov.names.x = NULL,
                    # lavaan options
-                   se         = "none", 
+                   se         = "none",
                    estimator  = "two.step",
                    # other options (for lavaan)
                    ...,
                    output = "cor") {
 
+    # shortcut if object = lavaan object
+    if(inherits(object, "lavaan")) {
+        out <- lav_cor_output(object, output = output)
+        return(out)
+    }
+
     # check estimator
     estimator <- tolower(estimator)
     if(estimator %in% c("two.step", "two.stage")) {
@@ -35,21 +41,14 @@ lavCor <- function(object,
     }
 
     # check object class
-    if(inherits(object, "lavaan")) {
-        lav.data <- object@Data
-        if(missing(missing)) {
-            missing <- object@Options$missing
-        } else {
-            missing <- "default"
-        }
-    } else if(inherits(object, "lavData")) {
+    if(inherits(object, "lavData")) {
         lav.data <- object
     } else if(inherits(object, "data.frame")) {
         NAMES <- names(object)
         if(!is.null(group)) {
             NAMES <- NAMES[- match(group, NAMES)]
         }
-        lav.data <- lavData(data = object, group = group, 
+        lav.data <- lavData(data = object, group = group,
                             ov.names = NAMES, ordered = ordered,
                             ov.names.x = ov.names.x,
                             lavoptions = list(missing = missing))
@@ -70,11 +69,13 @@ lavCor <- function(object,
 
     # extract partable options from dots
     dots <- list(...)
-    meanstructure <- FALSE; fixed.x <- FALSE; mimic <- "lavaan"
+    meanstructure <- FALSE
+    fixed.x       <- FALSE
+    mimic         <- "lavaan"
     conditional.x <- FALSE
     if(!is.null(dots$meanstructure)) {
         meanstructure <- dots$meanstructure
-    } 
+    }
     if(categorical) {
         meanstructure <- TRUE
     }
@@ -88,8 +89,13 @@ lavCor <- function(object,
         conditional.x <- dots$conditional.x
     }
 
+    # override, only for backwards compatibility (eg moments() in JWileymisc)
+    if(missing %in% c("ml", "fiml")) {
+        meanstructure = TRUE
+    }
+
     # generate partable for unrestricted model
-    PT.un <- 
+    PT.un <-
         lav_partable_unrestricted(lavobject     = NULL,
                                   lavdata       = lav.data,
                                   lavoptions    = list(meanstructure = meanstructure,
@@ -100,41 +106,64 @@ lavCor <- function(object,
                                   sample.mean   = NULL,
                                   sample.th     = NULL)
 
-    
-    fit <- lavaan(slotParTable = PT.un, slotData = lav.data,
+
+    FIT <- lavaan(slotParTable = PT.un, slotData = lav.data,
                   model.type = "unrestricted",
                   missing = missing,
                   se = se, estimator = estimator, ...)
 
+    out <- lav_cor_output(FIT, output = output)
+
+    out
+}
+
+lav_cor_output <- function(object, output = "cor") {
+
     # check output
     if(output %in% c("cor","cov")) {
-        out <- inspect(fit, "sampstat")
-        if(fit@Data@ngroups == 1L) {
-            out <- out$cov
+        out <- inspect(object, "sampstat")
+        if(object@Data@ngroups == 1L) {
+            if(object@Model@conditional.x) {
+                out <- out$res.cov
+            } else {
+                out <- out$cov
+            }
             if(output == "cor") {
                 out <- cov2cor(out)
             }
         } else {
-            out <- lapply(out, "[[", "cov")
+            if(object@Model@conditional.x) {
+                out <- lapply(out, "[[", "res.cov")
+            } else {
+                out <- lapply(out, "[[", "cov")
+            }
             if(output == "cor") {
                 out <- lapply(out, cov2cor)
             }
         }
     } else if(output %in% c("th","thresholds")) {
-        out <- inspect(fit, "sampstat")
-        if(fit@Data@ngroups == 1L) {
-            out <- out$th
+        out <- inspect(object, "sampstat")
+        if(object@Data@ngroups == 1L) {
+            if(object@Model@conditional.x) {
+                out <- out$res.th
+            } else {
+                out <- out$th
+            }
         } else {
-            out <- lapply(out, "[[", "th")
+            if(object@Model@conditional.x) {
+                out <- lapply(out, "[[", "res.th")
+            } else {
+                out <- lapply(out, "[[", "th")
+            }
         }
     } else if(output %in% c("sampstat")) {
-        out <- inspect(fit, "sampstat")
-    } else if(output %in% c("parameterEstimates", "pe", 
+        out <- inspect(object, "sampstat")
+    } else if(output %in% c("parameterEstimates", "pe",
               "parameterestimates", "est")) {
-        #out <- parameterEstimates(fit)
-        out <- standardizedSolution(fit)
+        #out <- parameterEstimates(object)
+        out <- standardizedSolution(object)
     } else {
-        out <- fit
+        out <- object
     }
 
     out
