@@ -9,11 +9,13 @@
 #
 # initial version: YR 14 April 2012
 
-# YR 23 feb 2017: blocks/levels/groups, but everything is group-based!
+# YR 23 Feb 2017: blocks/levels/groups, but everything is group-based!
 
 # FIXME: if nlevels > 1L, and ngroups > 1L, we should check that
 # group is at the upper-level
 
+# YR 08 May  2019: sampling weights normalization -> different options
+
 # extract the data we need for this particular model
 lavData <- function(data              = NULL,          # data.frame
                     group             = NULL,          # multiple groups?
@@ -87,7 +89,7 @@ lavData <- function(data              = NULL,          # data.frame
             if(is.matrix(data)) {
                 if(nrow(data) == ncol(data)) {
                     # perhaps it is a covariance matrix?
-                    if(data[2,1] == data[1,2] && warn) { # not perfect...
+                    if(isSymmetric(data)) {
                         warning("lavaan WARNING: data argument looks like a covariance matrix; please use the sample.cov argument instead")
                     }
                 }
@@ -96,7 +98,8 @@ lavData <- function(data              = NULL,          # data.frame
                 ### data matrices directly
                 data <- as.data.frame(data, stringsAsFactors = FALSE)
             } else {
-                stop("lavaan ERROR: data object of class ", class(data))
+                stop("lavaan ERROR: data= argument",
+                     " is not a data.fame, but of class ", sQuote(class(data)))
             }
         }
 
@@ -123,6 +126,8 @@ lavData <- function(data              = NULL,          # data.frame
                                  ov.names          = ov.names,
                                  ordered           = ordered,
                                  sampling.weights  = sampling.weights,
+                                 sampling.weights.normalization =
+                                     lavoptions$sampling.weights.normalization,
                                  ov.names.x        = ov.names.x,
                                  ov.names.l        = ov.names.l,
                                  std.ov            = std.ov,
@@ -144,6 +149,15 @@ lavData <- function(data              = NULL,          # data.frame
             stop("lavaan ERROR: please specify number of observations")
         }
 
+        # if a 'group' argument was provided, keep it -- new in 0.6-4
+        if(is.null(group)) {
+            group <- character(0L)
+        } else if(is.character(group)) {
+            # nothing to do, just store it
+        } else {
+            stop("lavaan ERROR: group argument should be a string")
+        }
+
         # list?
         if(is.list(sample.cov)) {
             # multiple groups, multiple cov matrices
@@ -157,10 +171,11 @@ lavData <- function(data              = NULL,          # data.frame
             ngroups     <- length(sample.cov)
             LABEL <- names(sample.cov)
             if(is.null(group.label) || length(group.label) == 0L) {
-                if(is.null(LABEL))
+                if(is.null(LABEL)) {
                     group.label <- paste("Group ", 1:ngroups, sep="")
-                else
+                } else {
                     group.label <- LABEL
+                }
             } else {
                 if(is.null(LABEL)) {
                     stopifnot(length(group.label) == ngroups)
@@ -231,7 +246,7 @@ lavData <- function(data              = NULL,          # data.frame
         lavData <- new("lavData",
                        data.type   = "moment",
                        ngroups     = ngroups,
-                       group       = character(0L),
+                       group       = group,
                        nlevels     = 1L, # for now
                        cluster     = character(0L),
                        group.label = group.label,
@@ -403,6 +418,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
                                                          # in model
                           ordered       = NULL,          # ordered variables
                           sampling.weights = NULL,       # sampling weights
+                          sampling.weights.normalization = "none",
                           ov.names.x    = character(0L), # exo variables
                           ov.names.l    = list(),        # var per level
                           std.ov        = FALSE,         # standardize ov's?
@@ -717,7 +733,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
                 nobs[[g]] <- length(case.idx[[g]])
                 norig[[g]] <- length(which(data[[group]] == group.label[g]))
                 if(warn && (nobs[[g]] < norig[[g]])) {
-                    warning("lavaan WARNING: ", (nobs[[g]] - norig[[g]]),
+                    warning("lavaan WARNING: ", (norig[[g]] - nobs[[g]]),
                         " cases were deleted in group ", group.label[g],
                         " due to missing values in ",
                         "\n\t\t  exogenous variable(s), while fixed.x = TRUE.")
@@ -754,6 +770,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
         X[[g]] <- data.matrix( data[case.idx[[g]], ov.idx, drop = FALSE] )
         dimnames(X[[g]]) <- NULL ### copy?
 
+        # sampling weights (but no normalization yet)
         if(!is.null(sampling.weights)) {
             WT <- data[[sampling.weights]][case.idx[[g]]]
             if(any(WT < 0)) {
@@ -767,9 +784,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
                         " contains missing values\n", sep = "")
             }
 
-            # rescale, so sum equals sample size in this group
-            WT2 <- WT / sum(WT) * nobs[[g]]
-            weights[[g]] <- WT2
+            weights[[g]] <- WT
         }
 
         # construct integers for user-declared 'ordered' factors
@@ -814,7 +829,8 @@ lav_data_full <- function(data          = NULL,          # data.frame
                            sort.freq = TRUE, coverage = TRUE)
             # checking!
             if(length(Mp[[g]]$empty.idx) > 0L) {
-                empty.case.idx <- Mp[[g]]$empty.idx
+                # new in 0.6-4: we return 'original' index in full data.frame
+                empty.case.idx <- case.idx[[g]][ Mp[[g]]$empty.idx ]
                 if(warn) {
                     warning("lavaan WARNING: some cases are empty and will be ignored:\n  ", paste(empty.case.idx, collapse=" "))
                 }
@@ -855,14 +871,122 @@ lav_data_full <- function(data          = NULL,          # data.frame
                                                  multilevel = multilevel,
                                                  ov.names = ov.names[[g]],
                                                  ov.names.l = ov.names.l[[g]])
-        }
+
+            # new in 0.6-4
+            # check for 'level-1' variables with zero within variance
+            l1.idx <- c(Lp[[g]]$within.idx[[2]], # within only
+                        Lp[[g]]$both.idx[[2]])
+            for(v in l1.idx) {
+                within.var <- tapply(X[[g]][,v], Lp[[g]]$cluster.idx[[2]],
+                                     FUN = var, na.rm = TRUE)
+                # ignore singletons
+                singleton.idx <- which( Lp[[g]]$cluster.size[[2]] == 1L )
+                if(length(singleton.idx) > 0L) {
+                    within.var[singleton.idx] <- 10 # non-zero variance
+                }
+                zero.var <- which(within.var < .Machine$double.eps)
+                if(length(zero.var) == 0L) {
+                    # all is good
+                } else if(length(zero.var) == length(within.var)) {
+                    # all zero! possibly a between-level variable
+                    gtxt <- if(ngroups > 1L) {
+                                paste(" in group ", g, ".", sep = "")
+                            } else { "." }
+                    txt <- c("Level-1 variable ", dQuote(ov.names[[g]][v]),
+                             " has no variance at the within level", gtxt,
+                             " The variable appears to be a between-level
+                             variable. Please remove this variable from
+                             the level 1 section in the model syntax.")
+                    warning(lav_txt2message(txt))
+                } else {
+                    # some zero variances!
+                    gtxt <- if(ngroups > 1L) {
+                                paste(" in group ", g, ".", sep = "")
+                            } else { "." }
+                    txt <- c("Level-1 variable ", dQuote(ov.names[[g]][v]),
+                       " has no variance within some clusters", gtxt,
+                       " The cluster ids with zero within variance are:\n",
+                       paste( Lp[[g]]$cluster.id[[2]][zero.var],
+                              collapse = " "))
+                    warning(lav_txt2message(txt))
+                }
+            }
+
+            # new in 0.6-4
+            # check for 'level-2' only variables with non-zero within variance
+            l2.idx <- Lp[[g]]$between.idx[[2]] # between only
+            error.flag <- FALSE
+            for(v in l2.idx) {
+                within.var <- tapply(X[[g]][,v], Lp[[g]]$cluster.idx[[2]],
+                                     FUN = var, na.rm = TRUE)
+                non.zero.var <- which(unname(within.var) > .Machine$double.eps)
+                if(length(non.zero.var) == 0L) {
+                    # all is good
+                } else if(length(non.zero.var) == 1L) {
+                    # just one
+                    gtxt <- if(ngroups > 1L) {
+                                paste(" in group ", g, ".", sep = "")
+                            } else { "." }
+                    txt <- c("Level-2 variable ", dQuote(ov.names[[g]][v]),
+                             " has non-zero variance at the within level", gtxt,
+                             " in one cluster with id: ",
+                               Lp[[g]]$cluster.id[[2]][non.zero.var], ".\n",
+                             " Please double-check if this is a between only",
+                             " variable.")
+                    warning(lav_txt2message(txt))
+                } else {
+                    error.flag <- TRUE
+                    # several
+                    gtxt <- if(ngroups > 1L) {
+                                paste(" in group ", g, ".", sep = "")
+                            } else { "." }
+                    txt <- c("Level-2 variable ", dQuote(ov.names[[g]][v]),
+                       " has non-zero variance at the within level", gtxt,
+                       " The cluster ids with non-zero within variance are: ",
+                           paste( Lp[[g]]$cluster.id[[2]][non.zero.var],
+                              collapse = " "))
+                    warning(lav_txt2message(txt))
+                }
+
+            }
+            if(error.flag) {
+                txt <- c("Some between-level (only) variables have non-zero ",
+                         " variance at the within-level. ",
+                         " Please double-check your data. ")
+                stop(lav_txt2message(txt, header = "lavaan ERROR"))
+            }
+
+        } # clustered data
 
     } # groups, at first level
 
+    # sampling weigths, again
     if(is.null(sampling.weights)) {
         sampling.weights <- character(0L)
+    } else {
+        # check if we need normalization
+        if(sampling.weights.normalization == "none") {
+            # nothing to do
+        } else if(sampling.weights.normalization == "total") {
+            sum.weights <- sum(unlist(weights))
+            ntotal <- sum(unlist(nobs))
+            for(g in 1:ngroups) {
+                WT <- weights[[g]]
+                WT2 <- WT / sum.weights * ntotal
+                weights[[g]] <- WT2
+            }
+        } else if(sampling.weights.normalization == "group") {
+            for(g in 1:ngroups) {
+                WT <- weights[[g]]
+                WT2 <- WT / sum(WT) * nobs[[g]]
+                weights[[g]] <- WT2
+            }
+        } else {
+            stop("lavaan ERROR: sampling.weights.normalization should be total, group or none.")
+        }
     }
 
+
     lavData <- new("lavData",
                    data.type       = "full",
                    ngroups         = ngroups,
@@ -1111,107 +1235,104 @@ function(object) {
     lav_data_print_short(object)
 })
 
-lav_data_print_short <- function(object) {
+lav_data_print_short <- function(object, nd = 3L) {
 
     lavdata <- object
+    num.format  <- paste("%", max(8L, nd + 5L), ".", nd, "f", sep = "")
 
     # listwise deletion?
-    listwise <- FALSE
+    listwise <- twocolumn <- FALSE
     for(g in 1:lavdata@ngroups) {
        if(lavdata@nobs[[1L]] != lavdata@norig[[1L]]) {
-           listwise <- TRUE
+           listwise <- twocolumn <- TRUE
            break
        }
     }
 
+    # header? no, for historical reasons only
     #cat("Data information:\n\n")
+
+    c1 <- c2 <- c3 <- character(0L)
+
+    # number of observations
     if(lavdata@ngroups == 1L) {
         if(listwise) {
-            cat(sprintf("  %-40s", ""), sprintf("  %10s", "Used"),
-                                        sprintf("  %10s", "Total"),
-                "\n", sep="")
+            c1 <- c(c1, ""); c2 <- c(c2, "Used"); c3 <- c(c3, "Total")
         }
-        t0.txt <- sprintf("  %-40s", "Number of observations")
-        t1.txt <- sprintf("  %10i", lavdata@nobs[[1L]])
-        t2.txt <- ifelse(listwise,
-                  sprintf("  %10i", lavdata@norig[[1L]]), "")
-        cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        c1 <- c(c1, "Number of observations")
+        c2 <- c(c2, lavdata@nobs[[1L]])
+        c3 <- c(c3, ifelse(listwise, lavdata@norig[[1L]], ""))
+    } else {
+        c1 <- c(c1, "Number of observations per group:");
+        if(listwise) {
+            c2 <- c(c2, "Used"); c3 <- c(c3, "Total")
+        } else {
+            c2 <- c(c2, ""); c3 <- c(c3, "")
+        }
+        for(g in 1:lavdata@ngroups) {
+            c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
+            c2 <- c(c2, lavdata@nobs[[g]])
+            c3 <- c(c3, ifelse(listwise, lavdata@norig[[g]], ""))
+        } # g
+    }
 
+    # number of clusters
+    if(lavdata@ngroups == 1L) {
         if( (.hasSlot(lavdata, "nlevels")) && # in case we have an old obj
             (lavdata@nlevels > 1L) ) {
-            #cat("\n")
             for(l in 2:lavdata@nlevels) {
-                t0.txt <- sprintf("  %-40s",
-                    paste("Number of clusters [", lavdata@cluster[l-1], "]",
-                          sep = ""))
-                t1.txt <- sprintf("  %10i", lavdata@Lp[[1]]$nclusters[[l]])
-                #t2.txt <- ifelse(listwise,
-                #          sprintf("  %10i", lavdata@norig[[1L]]), "")
-                t2.txt <- ""
-                cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+                c1 <- c(c1,
+                        paste("Number of clusters [", lavdata@cluster[l-1], "]",
+                        sep = ""))
+                c2 <- c(c2, lavdata@Lp[[1]]$nclusters[[l]])
+                c3 <- c(c3, "")
             }
         } else if( (.hasSlot(lavdata, "cluster")) &&
                    (length(lavdata@cluster) > 0L) ) {
-            t0.txt <- sprintf("  %-40s",
-            paste("Number of clusters [", lavdata@cluster, "]", sep = ""))
-            t1.txt <- sprintf("  %10i", lavdata@Lp[[1]]$nclusters[[2]])
-            t2.txt <- ""
-            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+            c1 <- c(c1, paste("Number of clusters [", lavdata@cluster, "]",
+                              sep = ""))
+            c2 <- c(c2, lavdata@Lp[[1]]$nclusters[[2]])
+            c3 <- c(c3, "")
         }
     } else {
-        if(listwise) {
-            cat(sprintf("  %-40s", ""), sprintf("  %10s", "Used"),
-                                       sprintf("  %10s", "Total"),
-               "\n", sep="")
-        }
-        t0.txt <- sprintf("  %-40s", "Number of observations per group")
-        cat(t0.txt, "\n")
-        for(g in 1:lavdata@ngroups) {
-            t.txt <- sprintf("  %-40s  %10i", lavdata@group.label[[g]],
-                                              lavdata@nobs[[g]])
-            t2.txt <- ifelse(listwise,
-                      sprintf("  %10i", lavdata@norig[[g]]), "")
-            cat(t.txt, t2.txt, "\n", sep="")
-
-            if( (.hasSlot(lavdata, "nlevels")) &&
-                (lavdata@nlevels > 1L) ) {
-                #cat("\n")
-                for(l in 2:lavdata@nlevels) {
-                    t0.txt <- sprintf("  %-40s",
-                        paste("Number of clusters [", lavdata@cluster[l-1], "]",
-                              sep = ""))
-                    t1.txt <- sprintf("  %10i",
-                                      lavdata@Lp[[g]]$nclusters[[l]])
-                    #t2.txt <- ifelse(listwise,
-                    #          sprintf("  %10i", lavdata@norig[[1L]]), "")
-                    t2.txt <- ""
-                    cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        if( (.hasSlot(lavdata, "nlevels")) && (lavdata@nlevels > 1L) ) {
+            for(l in 2:lavdata@nlevels) {
+                c1 <- c(c1,
+                  paste("Number of clusters [", lavdata@cluster[l-1], "]:",
+                        sep = ""))
+                c2 <- c(c2, ""); c3 <- c(c3, "")
+                for(g in 1:lavdata@ngroups) {
+                    c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
+                    c2 <- c(c2, lavdata@Lp[[g]]$nclusters[[l]])
+                    c3 <- c(c3, "")
                 }
-            } else if( (.hasSlot(lavdata, "cluster")) &&
-                   (length(lavdata@cluster) > 0L) ) {
-                t0.txt <- sprintf("  %-40s",
-                paste("Number of clusters [", lavdata@cluster, "]", sep = ""))
-                t1.txt <- sprintf("  %10i", lavdata@Lp[[g]]$nclusters[[2]])
-                t2.txt <- ""
-                cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
             }
-        } # g
+        } else if( (.hasSlot(lavdata, "cluster")) &&
+               (length(lavdata@cluster) > 0L) ) {
+            c1 <- c(c1,
+             paste("Number of clusters [", lavdata@cluster, "]:", sep = ""))
+            c2 <- c(c2, ""); c3 <- c(c3, "")
+            for(g in 1:lavdata@ngroups) {
+                c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
+                c2 <- c(c2, lavdata@Lp[[g]]$nclusters[[2]])
+                c3 <- c(c3, "")
+            }
+        }
     }
 
     # missing patterns?
     if(!is.null(lavdata@Mp[[1L]])) {
         if(lavdata@ngroups == 1L) {
-            t0.txt <- sprintf("  %-40s", "Number of missing patterns")
-            t1.txt <- sprintf("  %10i",
-                              lavdata@Mp[[1L]]$npatterns)
-            cat(t0.txt, t1.txt, "\n", sep="")
+            c1 <- c(c1, "Number of missing patterns")
+            c2 <- c(c2, lavdata@Mp[[1L]]$npatterns)
+            c3 <- c(c3, "")
         } else {
-            t0.txt <- sprintf("  %-40s", "Number of missing patterns per group")
-            cat(t0.txt, "\n")
+            c1 <- c(c1, "Number of missing patterns per group:")
+            c2 <- c(c2, ""); c3 <- c(c3, "")
             for(g in 1:lavdata@ngroups) {
-                t.txt <- sprintf("  %-40s  %10i", lavdata@group.label[[g]],
-                                 lavdata@Mp[[g]]$npatterns)
-                cat(t.txt, "\n", sep="")
+                c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
+                c2 <- c(c2, lavdata@Mp[[g]]$npatterns)
+                c3 <- c(c3, "")
             }
         }
     }
@@ -1219,11 +1340,31 @@ lav_data_print_short <- function(object) {
     # sampling weights?
     if( (.hasSlot(lavdata, "weights")) && # in case we have an old object
         (!is.null(lavdata@weights[[1L]])) ) {
-        t0.txt <- sprintf("  %-30s", "Sampling weights variable")
-        t1.txt <- sprintf("  %20s", lavdata@sampling.weights)
-        cat(t0.txt, t1.txt, "\n", sep="")
+        c1 <- c(c1, "Sampling weights variable")
+        c2 <- c(c2, lavdata@sampling.weights)
+        c3 <- c(c3, "")
     }
 
-    cat("\n")
+    # empty row
+    c1 <- c(c1, ""); c2 <- c(c2, ""); c3 <- c(c3, "")
+
+    # format c1/c2
+    c1 <- format(c1, width = 43L)
+    c2 <- format(c2, width = 8L + max(0, (nd - 3L)) * 4L, justify = "right")
+    c3 <- format(c3, width = 8L + nd, justify = "right")
+
+    # create character matrix
+    if(twocolumn) {
+        M <- cbind(c1, c2, c3, deparse.level = 0)
+    } else {
+        M <- cbind(c1, c2, deparse.level = 0)
+    }
+    colnames(M) <- rep("",  ncol(M))
+    rownames(M) <- rep(" ", nrow(M))
+
+    # print
+    write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
+
+    invisible(M)
 }
 
