@@ -1,13 +1,12 @@
 # model gradient
 
-lav_model_gradient <- function(lavmodel       = NULL, 
-                               GLIST          = NULL, 
-                               lavsamplestats = NULL, 
+lav_model_gradient <- function(lavmodel       = NULL,
+                               GLIST          = NULL,
+                               lavsamplestats = NULL,
                                lavdata        = NULL,
-                               lavcache       = NULL, 
-                               type           = "free", 
-                               verbose        = FALSE, 
-                               forcePD        = TRUE, 
+                               lavcache       = NULL,
+                               type           = "free",
+                               verbose        = FALSE,
                                group.weight   = TRUE,
                                Delta          = NULL,
                                m.el.idx       = NULL,
@@ -55,7 +54,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
         # compute moments for all groups
         #if(conditional.x) {
         #    Sigma.hat <- computeSigmaHatJoint(lavmodel = lavmodel,
-        #                     GLIST = GLIST, lavsamplestats = lavsamplestats,
+        #                     GLIST = GLIST,
         #                     extra = (estimator %in% c("ML", "REML","NTRLS")))
         #} else {
             Sigma.hat <- computeSigmaHat(lavmodel = lavmodel, GLIST = GLIST,
@@ -63,26 +62,24 @@ lav_model_gradient <- function(lavmodel       = NULL,
         #}
 
         # ridge here?
-        if(meanstructure && !categorical) {
+        if(meanstructure) {
             #if(conditional.x) {
-            #    Mu.hat <- computeMuHat(lavmodel = lavmodel, GLIST = GLIST,
-            #                           lavsamplestats = lavsamplestats)
-            #} else { 
+            #    Mu.hat <- computeMuHat(lavmodel = lavmodel, GLIST = GLIST)
+            #} else {
                 Mu.hat <- computeMuHat(lavmodel = lavmodel, GLIST = GLIST)
             #}
-        } else if(categorical) {
+        }
+
+        if(categorical) {
             TH <- computeTH(lavmodel = lavmodel, GLIST = GLIST)
         }
+
         if(conditional.x) {
             PI <- computePI(lavmodel = lavmodel, GLIST = GLIST)
+        } else if(estimator == "PML") {
+            PI <- vector("list", length = lavmodel@nblocks)
         }
-        if(estimator == "PML") {
-            if(lavmodel@nexo > 0L) {
-                PI <- computePI(lavmodel = lavmodel)
-            } else {
-                PI <- vector("list", length = lavmodel@nblocks)
-            }
-        }
+
         if(group.w.free) {
             GW <- computeGW(lavmodel = lavmodel, GLIST = GLIST)
         }
@@ -101,19 +98,20 @@ lav_model_gradient <- function(lavmodel       = NULL,
     # - PML/FML/MML: custom
 
     # 1. ML approach
-    if( (estimator == "ML" || estimator == "REML") && 
+    if( (estimator == "ML" || estimator == "REML") &&
+        lavdata@nlevels == 1L &&
         !lavmodel@conditional.x ) {
 
         if(meanstructure) {
             Omega <- computeOmega(Sigma.hat=Sigma.hat, Mu.hat=Mu.hat,
-                                  lavsamplestats=lavsamplestats, 
-                                  estimator=estimator, 
-                                  meanstructure=TRUE, 
+                                  lavsamplestats=lavsamplestats,
+                                  estimator=estimator,
+                                  meanstructure=TRUE,
                                   conditional.x = conditional.x)
             Omega.mu <- attr(Omega, "mu")
         } else {
             Omega <- computeOmega(Sigma.hat=Sigma.hat, Mu.hat=NULL,
-                                  lavsamplestats=lavsamplestats, 
+                                  lavsamplestats=lavsamplestats,
                                   estimator=estimator,
                                   meanstructure=FALSE,
                                   conditional.x = conditional.x)
@@ -122,14 +120,14 @@ lav_model_gradient <- function(lavmodel       = NULL,
 
         # compute DX (for all elements in every model matrix)
         DX <- vector("list", length=length(GLIST))
-      
+
         for(g in 1:lavmodel@nblocks) {
             # which mm belong to group g?
             mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
             mm.names <- names( GLIST[mm.in.group] )
 
             if(representation == "LISREL") {
-                DX.group <- derivative.F.LISREL(GLIST[mm.in.group], 
+                DX.group <- derivative.F.LISREL(GLIST[mm.in.group],
                                                 Omega[[g]],
                                                 Omega.mu[[g]])
 
@@ -140,7 +138,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
                 }
 
                 # only save what we need
-                DX[mm.in.group] <- DX.group[ mm.names ] 
+                DX[mm.in.group] <- DX.group[ mm.names ]
             } else {
                 stop("only representation LISREL has been implemented for now")
             }
@@ -173,7 +171,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
             dx <- DX
             # handle equality constraints
             ### FIXME!!!! TODO!!!!
-        } 
+        }
 
     } else # ML
 
@@ -192,8 +190,8 @@ lav_model_gradient <- function(lavmodel       = NULL,
             #diff <- as.matrix(lavsamplestats@WLS.obs[[g]]  - WLS.est[[g]])
             #group.dx <- -1 * ( t(Delta[[g]]) %*% lavsamplestats@WLS.V[[g]] %*% diff)
             # 0.5-17: use crossprod twice; treat DWLS/ULS special
-            if(estimator == "WLS" || 
-               estimator == "GLS" || 
+            if(estimator == "WLS" ||
+               estimator == "GLS" ||
                estimator == "NTRLS") {
                 # full weight matrix
                 diff <- lavsamplestats@WLS.obs[[g]]  - WLS.est[[g]]
@@ -201,7 +199,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
                 # full weight matrix
                 if(estimator == "GLS" || estimator == "WLS") {
                     WLS.V <- lavsamplestats@WLS.V[[g]]
-                    group.dx <- -1 * crossprod(Delta[[g]], 
+                    group.dx <- -1 * crossprod(Delta[[g]],
                                                crossprod(WLS.V, diff))
                 } else if(estimator == "NTRLS") {
                     stopifnot(!conditional.x)
@@ -238,7 +236,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
                     group.dx <- as.numeric( -1 * crossprod(Delta[[g]], POST) )
                 }
 
-            } else 
+            } else
 
             if(estimator == "DWLS" || estimator == "ULS") {
                 # diagonal weight matrix
@@ -259,9 +257,9 @@ lav_model_gradient <- function(lavmodel       = NULL,
             # nothing to do
         } else {
             # make a GLIST
-            dx <- lav_model_x2GLIST(lavmodel = lavmodel, x = dx, 
+            dx <- lav_model_x2GLIST(lavmodel = lavmodel, x = dx,
                                     type = "custom", setDelta = FALSE,
-                                    m.el.idx = m.el.idx, 
+                                    m.el.idx = m.el.idx,
                                     x.el.idx = x.el.idx)
         }
 
@@ -290,12 +288,12 @@ lav_model_gradient <- function(lavmodel       = NULL,
             Sigma.inv <- attr(Sigma, "inv")
             nvar  <- NROW(Sigma)
             S <- lavsamplestats@res.cov[[g]]
-    
+
             # beta
             OBS <- t( cbind(lavsamplestats@res.int[[g]],
                             lavsamplestats@res.slopes[[g]]) )
             EST <- t( cbind(Mu.g, PI.g) )
-            #obs.beta <- c(lavsamplestats@res.int[[g]], 
+            #obs.beta <- c(lavsamplestats@res.int[[g]],
             #              lav_matrix_vec(lavsamplestats@res.slopes[[g]]))
             #est.beta <- c(Mu.g,  lav_matrix_vec(PI.g))
             #beta.COV <- C3 %x% Sigma.inv
@@ -312,16 +310,18 @@ lav_model_gradient <- function(lavmodel       = NULL,
             #POST.beta <- 2 *  beta.COV %*% (obs.beta - est.beta)
             d.BETA <- C3 %*% (OBS - EST) %*% Sigma.inv
             # NOTE: the vecr here, unlike lav_mvreg_dlogl_beta
-            #       this is because DELTA has used vec(t(BETA)), 
+            #       this is because DELTA has used vec(t(BETA)),
             #       instead of vec(BETA)
-            POST.beta <- 2 * lav_matrix_vecr(d.BETA)
+            #POST.beta <- 2 * lav_matrix_vecr(d.BETA)
+            # NOT any longer, since 0.6-1!!!
+            POST.beta <- 2 * lav_matrix_vec(d.BETA)
 
             #POST.sigma1 <- lav_matrix_duplication_pre(
             #        (Sigma.inv %x% Sigma.inv)  %*% t(AB)  %*% (t(a) %x% b) )
 
             # Sigma
             #POST.sigma2 <- lav_matrix_duplication_pre(
-            #                 matrix( lav_matrix_vec( 
+            #                 matrix( lav_matrix_vec(
             #          Sigma.inv %*% (S - Sigma) %*% t(Sigma.inv)), ncol = 1L))
             W.tilde <- S + t(OBS - EST) %*% C3 %*% (OBS - EST)
             d.SIGMA <- (Sigma.inv - Sigma.inv %*% W.tilde %*% Sigma.inv)
@@ -334,6 +334,9 @@ lav_model_gradient <- function(lavmodel       = NULL,
 
             group.dx <- as.numeric( -1 * crossprod(Delta[[g]], POST) )
 
+            # because we still use obj/2, we need to divide by 2!
+            group.dx <- group.dx / 2 # fixed in 0.6-1
+
             group.dx <- group.w[g] * group.dx
             if(g == 1) {
                 dx <- group.dx
@@ -346,13 +349,50 @@ lav_model_gradient <- function(lavmodel       = NULL,
             # nothing to do
         } else {
             # make a GLIST
-            dx <- lav_model_x2GLIST(lavmodel = lavmodel, x = dx, 
+            dx <- lav_model_x2GLIST(lavmodel = lavmodel, x = dx,
                                     type = "custom", setDelta = FALSE,
-                                    m.el.idx = m.el.idx, 
+                                    m.el.idx = m.el.idx,
                                     x.el.idx = x.el.idx)
         }
     } # ML + conditional.x
 
+    else if(estimator == "ML" && lavdata@nlevels > 1L) {
+        if(type != "free") {
+            stop("FIXME: type != free in lav_model_gradient for estimator ML for nlevels > 1")
+        } else {
+            Delta <- computeDelta(lavmodel = lavmodel, GLIST. = GLIST)
+        }
+
+        # for each upper-level group....
+        for(g in 1:lavmodel@ngroups) {
+            DX <- lav_mvnorm_cluster_dlogl_2l_samplestats(
+                       YLp = lavsamplestats@YLp[[g]],
+                       Lp  = lavdata@Lp[[g]],
+                       Mu.W    = Mu.hat[[(g-1)*2 + 1]],
+                       Sigma.W = Sigma.hat[[(g-1)*2 + 1]],
+                       Mu.B    = Mu.hat[[(g-1)*2 + 2]],
+                       Sigma.B = Sigma.hat[[(g-1)*2 + 2]],
+                       Sinv.method  = "eigen")
+
+            group.dx <- as.numeric( DX %*% Delta[[g]] )
+
+            # group weights (if any)
+            group.dx <- group.w[g] * group.dx
+            if(g == 1) {
+                dx <- group.dx
+            } else {
+                dx <- dx + group.dx
+            }
+        } # g
+
+        # divide by 2 * N
+        dx <- dx / (2 * lavsamplestats@ntotal)
+
+        #cat("dx1 (numerical) = \n"); print( zapsmall(dx1) )
+        #cat("dx  (analytic)  = \n"); print( zapsmall(dx ) )
+
+    } # ML + two-level
+
     else if(estimator == "PML" || estimator == "FML" ||
             estimator == "MML") {
 
@@ -370,23 +410,43 @@ lav_model_gradient <- function(lavmodel       = NULL,
             #print(TH[[g]])
             #cat("*****\n")
 
-            # compute partial derivative of logLik with respect to 
+            # compute partial derivative of logLik with respect to
             # thresholds/means, slopes, variances, correlations
             if(estimator == "PML") {
-                d1 <- pml_deriv1(Sigma.hat  = Sigma.hat[[g]],
-                                 TH         = TH[[g]],
-                                 th.idx     = th.idx[[g]],
-                                 num.idx    = num.idx[[g]],
-                                 X          = lavdata@X[[g]],
-                                 lavcache   = lavcache[[g]],
-                                 eXo        = lavdata@eXo[[g]],
-                                 PI         = PI[[g]],
-                                 missing    = lavdata@missing)
+
+                if(lavdata@nlevels  > 1L) {
+                    stop("lavaan ERROR: PL gradient + multilevel not implemented; try optim.gradient = \"numerical\"")
+                } else if(conditional.x) {
+                    d1 <- pml_deriv1(Sigma.hat  = Sigma.hat[[g]],
+                                     Mu.hat     = Mu.hat[[g]],
+                                     TH         = TH[[g]],
+                                     th.idx     = th.idx[[g]],
+                                     num.idx    = num.idx[[g]],
+                                     X          = lavdata@X[[g]],
+                                     lavcache   = lavcache[[g]],
+                                     eXo        = lavdata@eXo[[g]],
+                                     PI         = PI[[g]],
+                                     missing    = lavdata@missing)
+                } else {
+                    d1 <- pml_deriv1(Sigma.hat  = Sigma.hat[[g]],
+                                     Mu.hat     = Mu.hat[[g]],
+                                     TH         = TH[[g]],
+                                     th.idx     = th.idx[[g]],
+                                     num.idx    = num.idx[[g]],
+                                     X          = lavdata@X[[g]],
+                                     lavcache   = lavcache[[g]],
+                                     eXo        = NULL,
+                                     PI         = NULL,
+                                     missing    = lavdata@missing)
+                } # not conditional.x
+
                 # chain rule (fmin)
-                group.dx <- 
+                group.dx <-
                     as.numeric(t(d1) %*% Delta[[g]])
 
-            } else if(estimator == "FML") {
+           } # PML
+
+           else if(estimator == "FML") {
                 d1 <- fml_deriv1(Sigma.hat = Sigma.hat[[g]],
                                  TH        = TH[[g]],
                                  th.idx    = th.idx[[g]],
@@ -395,11 +455,11 @@ lav_model_gradient <- function(lavmodel       = NULL,
                                  lavcache  = lavcache[[g]])
 
                 # chain rule (fmin)
-                group.dx <- 
+                group.dx <-
                     as.numeric(t(d1) %*% Delta[[g]])/lavsamplestats@nobs[[g]]
 
             } else if(estimator == "MML") {
-                group.dx <- 
+                group.dx <-
                     lav_model_gradient_mml(lavmodel    = lavmodel,
                                     GLIST          = GLIST,
                                     THETA          = THETA[[g]],
@@ -426,7 +486,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
 
 
     # group.w.free for ML
-    if(lavmodel@group.w.free && 
+    if(lavmodel@group.w.free &&
        estimator %in% c("ML","MML","FML","PML","REML")) {
         #est.prop <- unlist( computeGW(lavmodel = lavmodel, GLIST = GLIST) )
         #obs.prop <- unlist(lavsamplestats@group.w)
@@ -442,7 +502,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
 
         # remove last element (fixed LAST group to zero)
         # dx.GW <- dx.GW[-length(dx.GW)]
-        
+
         # fill in in dx
         gw.mat.idx <- which(names(lavmodel@GLIST) == "gw")
         gw.x.idx <- unlist( lavmodel@x.free.idx[gw.mat.idx] )
@@ -462,7 +522,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
 #
 #    # state or final?
 #   if(is.null(GLIST)) GLIST <- lavmodel@GLIST
-#   
+#
 #   compute.moments <- function(x) {
 #       GLIST <- lav_model_x2GLIST(lavmodel = NULL, x=x, type="free")
 #       Sigma.hat <- computeSigmaHat(lavmodel = NULL, GLIST = GLIST)
@@ -470,7 +530,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
 #        if(lavmodel@meanstructure) {
 #            Mu.hat <- computeMuHat(lavmodel = NULL, GLIST=GLIST)
 #            out <- c(Mu.hat[[g]], S.vec)
-#        } else {   
+#        } else {
 #            out <- S.vec
 #        }
 #        out
@@ -486,7 +546,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
 ### FIXME: should we here also:
 ###        - weight for groups? (no, for now)
 ###        - handle equality constraints? (yes, for now)
-computeDelta <- function(lavmodel = NULL, GLIST. = NULL, 
+computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
                          m.el.idx. = NULL, x.el.idx. = NULL) {
 
     representation   <- lavmodel@representation
@@ -513,7 +573,7 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
     # type = "free" or something else?
     type <- "nonfree"
     m.el.idx <- m.el.idx.; x.el.idx <- x.el.idx.
-    if(is.null(m.el.idx) && is.null(x.el.idx)) 
+    if(is.null(m.el.idx) && is.null(x.el.idx))
          type <- "free"
 
     # number of rows in DELTA.group
@@ -556,12 +616,12 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
                     m.el.idx[[mm]] <- m.el.idx[[mm]][!dix]
                     x.el.idx[[mm]] <- x.el.idx[[mm]][!dix]
                 }
-            }    
+            }
         }
     } else {
         ## FIXME: this does *not* take into account symmetric
         ##        matrices; hence NCOL will be too large, and empty
-        ##        columns will be added 
+        ##        columns will be added
         ##        this is ugly, but it doesn't hurt
         ## alternative could be:
         ## NCOL <- sum(unlist(lapply(x.el.idx, function(x) length(unique(x)))))
@@ -581,7 +641,7 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
 
-        # label rows of Delta.group --- FIXME!!! 
+        # label rows of Delta.group --- FIXME!!!
         #if(categorical) {
         #    # 1. th (means interleaved?)
         #    # 2. pi
@@ -595,7 +655,7 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
 
         # if theta, do some preparation
         if(parameterization == "theta") {
-            sigma.hat <- computeSigmaHat.LISREL(MLIST=GLIST[mm.in.group], 
+            sigma.hat <- computeSigmaHat.LISREL(MLIST=GLIST[mm.in.group],
                                                 delta=FALSE)
             dsigma <- diag(sigma.hat)
             # dcor/dcov for sigma
@@ -621,40 +681,51 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
                 if(categorical && parameterization == "theta") {
                     DELTA <- R %*% DELTA
                 }
-                
+
                 if(categorical) {
                     # reorder: first variances (of numeric), then covariances
                     cov.idx  <- lav_matrix_vech_idx(nvar[g])
                     covd.idx <- lav_matrix_vech_idx(nvar[g], diagonal = FALSE)
 
-                    var.idx <- which(is.na(match(cov.idx, 
+                    var.idx <- which(is.na(match(cov.idx,
                                                  covd.idx)))[num.idx[[g]]]
                     cor.idx <- match(covd.idx, cov.idx)
- 
-                    DELTA <- rbind(DELTA[var.idx,,drop=FALSE], 
+
+                    DELTA <- rbind(DELTA[var.idx,,drop=FALSE],
                                    DELTA[cor.idx,,drop=FALSE])
                 }
+
                 if(!categorical) {
-                    if(lavmodel@meanstructure) {
+                    if(conditional.x && lavmodel@nexo[g] > 0L) {
+                        # ATTENTION: we need to change the order here
+                        # lav_mvreg_scores_* uses 'Beta' where the
+                        # the intercepts are just the first row
+                        # using the col-major approach, we need to
+                        # interweave the intercepts with the slopes!
                         DELTA.mu <- derivative.mu.LISREL(m=mname,
                                idx=m.el.idx[[mm]], MLIST=GLIST[ mm.in.group ])
-                        if(conditional.x && lavmodel@nexo[g] > 0L) {
-                            DELTA.pi <- derivative.pi.LISREL(m=mname,
+                        DELTA.pi <- derivative.pi.LISREL(m=mname,
                              idx=m.el.idx[[mm]], MLIST=GLIST[ mm.in.group ])
-                            DELTA <- rbind(DELTA.mu, DELTA.pi, DELTA)
-                        } else {
+
+                        nEls <- NROW(DELTA.mu) + NROW(DELTA.pi)
+                        # = (nexo + 1 int) * nvar
+
+                        # intercepts on top
+                        tmp <- rbind(DELTA.mu, DELTA.pi)
+                        # change row index
+                        row.idx <- lav_matrix_vec(matrix(seq.int(nEls),
+                            nrow = lavmodel@nexo[g] + 1L,
+                            ncol = lavmodel@nvar[g], byrow = TRUE))
+                        DELTA.beta <- tmp[row.idx,,drop = FALSE]
+                        DELTA <- rbind(DELTA.beta, DELTA)
+                    } else if(!conditional.x && lavmodel@meanstructure) {
+                            DELTA.mu <- derivative.mu.LISREL(m=mname,
+                               idx=m.el.idx[[mm]], MLIST=GLIST[ mm.in.group ])
                             DELTA <- rbind(DELTA.mu, DELTA)
-                        }
-                    } else {
-                        if(conditional.x && lavmodel@nexo[g] > 0L) {
-                            DELTA.pi <- derivative.pi.LISREL(m=mname,
-                             idx=m.el.idx[[mm]], MLIST=GLIST[ mm.in.group ])
-                            DELTA <- rbind(DELTA.pi, DELTA)
-                        } else {
-                            # nothing to do
-                        }
                     }
-                } else if(categorical) {
+                }
+
+                else if(categorical) {
                     DELTA.th <- derivative.th.LISREL(m=mname,
                                                      idx=m.el.idx[[mm]],
                                                      th.idx=th.idx[[g]],
@@ -662,22 +733,22 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
                                                      delta = TRUE)
                     if(parameterization == "theta") {
                         # dy/ddsigma = -0.5/(ddsigma*sqrt(ddsigma))
-                        dDelta.dx <- 
-                            ( dxSigma[theta.var.idx,,drop=FALSE] * 
+                        dDelta.dx <-
+                            ( dxSigma[theta.var.idx,,drop=FALSE] *
                               -0.5 / (dsigma*sqrt(dsigma)) )
-                        dth.dDelta <- 
+                        dth.dDelta <-
                             derivative.th.LISREL(m = "delta",
                                                  idx = 1:nvar[g],
                                                  MLIST = GLIST[ mm.in.group ],
                                                  th.idx = th.idx[[g]])
                         # add dth.dDelta %*% dDelta.dx
                         no.num.idx <- which(th.idx[[g]] > 0)
-                        DELTA.th[no.num.idx,] <- 
+                        DELTA.th[no.num.idx,] <-
                             DELTA.th[no.num.idx,,drop=FALSE] +
                             (dth.dDelta %*% dDelta.dx)[no.num.idx,,drop=FALSE]
                     }
-                    if(lavmodel@nexo[g] > 0L) {
-                        DELTA.pi <- 
+                    if(conditional.x && lavmodel@nexo[g] > 0L) {
+                        DELTA.pi <-
                             derivative.pi.LISREL(m=mname,
                                                  idx=m.el.idx[[mm]],
                                                  MLIST=GLIST[ mm.in.group ])
@@ -687,11 +758,11 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
                                     idx = 1:nvar[g],
                                     MLIST = GLIST[ mm.in.group ])
                             # add dpi.dDelta %*% dDelta.dx
-                            no.num.idx <- 
+                            no.num.idx <-
                                 which(!seq.int(1L,nvar[g]) %in% num.idx[[g]])
                             no.num.idx <- rep(seq.int(0,nexo[g]-1) * nvar[g],
                                           each=length(no.num.idx)) + no.num.idx
-                            DELTA.pi[no.num.idx,] <- 
+                            DELTA.pi[no.num.idx,] <-
                                 DELTA.pi[no.num.idx,,drop=FALSE] +
                                 (dpi.dDelta %*% dDelta.dx)[no.num.idx,,drop=FALSE]
                         }
@@ -719,7 +790,7 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
         #if(type == "free" && lavmodel@eq.constraints) {
         #    Delta.group <- Delta.group %*% lavmodel@eq.constraints.K
         #}
-  
+
         #Delta.eq <- Delta.group
         # save(Delta.eq, file=paste0("delta_NO_EQ",g,".Rdata"))
 
@@ -727,6 +798,16 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
 
     } # g
 
+    # if multilevel, rbind levels within group
+    if(.hasSlot(lavmodel, "multilevel") && lavmodel@multilevel) {
+        DELTA <- vector("list", length = lavmodel@ngroups)
+        for(g in 1:lavmodel@ngroups) {
+            DELTA[[g]] <- rbind( Delta[[(g-1)*2 + 1]],
+                                 Delta[[(g-1)*2 + 2]] )
+        }
+        Delta <- DELTA
+    }
+
     Delta
 }
 
@@ -794,7 +875,7 @@ computeDeltaDx <- function(lavmodel = NULL, GLIST = NULL, target = "lambda") {
                     DELTA <- derivative.tau.LISREL(m=mname,
                                idx=m.el.idx[[mm]], MLIST=GLIST[ mm.in.group ])
                 } else if(target == "theta") {
-                    DELTA <- derivative.theta.LISREL(m=mname, 
+                    DELTA <- derivative.theta.LISREL(m=mname,
                                idx=m.el.idx[[mm]], MLIST=GLIST[ mm.in.group ])
                 } else if(target == "gamma") {
                     DELTA <- derivative.gamma.LISREL(m=mname,
@@ -834,8 +915,8 @@ computeDeltaDx <- function(lavmodel = NULL, GLIST = NULL, target = "lambda") {
     Delta
 }
 
-computeOmega <- function(Sigma.hat=NULL, Mu.hat=NULL,  
-                         lavsamplestats=NULL, estimator="ML", 
+computeOmega <- function(Sigma.hat=NULL, Mu.hat=NULL,
+                         lavsamplestats=NULL, estimator="ML",
                          meanstructure=FALSE, conditional.x = FALSE) {
 
     # nblocks
@@ -871,7 +952,7 @@ computeOmega <- function(Sigma.hat=NULL, Mu.hat=NULL,
                     }
                     # Browne 1995 eq 4.55
                     Omega.mu[[g]] <- t(t(diff) %*% Sigma.hat.inv)
-                    Omega[[g]] <- 
+                    Omega[[g]] <-
                         ( Sigma.hat.inv %*% (W.tilde - Sigma.hat[[g]]) %*%
                           Sigma.hat.inv )
                 } else {
@@ -880,7 +961,7 @@ computeOmega <- function(Sigma.hat=NULL, Mu.hat=NULL,
                     } else {
                         W.tilde <- lavsamplestats@cov[[g]]
                     }
-                    Omega[[g]] <- 
+                    Omega[[g]] <-
                         ( Sigma.hat.inv %*% (W.tilde - Sigma.hat[[g]]) %*%
                           Sigma.hat.inv )
                 }
@@ -962,7 +1043,7 @@ lav_model_gradient_DD <- function(lavmodel, GLIST = NULL, group = 1L) {
     # fix Delta's...
     mm.in.group <- 1:lavmodel@nmat[group] + cumsum(c(0,lavmodel@nmat))[group]
     MLIST <- GLIST[ mm.in.group ]
-    
+
     DD <- list()
     nvar <- lavmodel@nvar
     nfac <- ncol(MLIST$lambda) - length(lv.dummy.idx)
@@ -995,7 +1076,7 @@ lav_model_gradient_DD <- function(lavmodel, GLIST = NULL, group = 1L) {
         beta.idx <- rep(nr*((1:nc) - 1L), times=length(lv.dummy.idx)) + rep(lv.dummy.idx, each=nc)
 
         #l.idx <- inr*((1:nc) - 1L) + rep(ov.dummy.idx, each=nc) ## FIXME
-        # l.idx <- rep(nr*((1:nc) - 1L), each=length(ov.dummy.idx)) + rep(ov.dummy.idx, times=nc) 
+        # l.idx <- rep(nr*((1:nc) - 1L), each=length(ov.dummy.idx)) + rep(ov.dummy.idx, times=nc)
         l.idx <- rep(nr*((1:nc) - 1L), times=length(ov.dummy.idx)) + rep(ov.dummy.idx, each=nc)
         DD$lambda[match(l.idx, lambda.idx),] <- Delta.beta[beta.idx,,drop=FALSE]
     }
@@ -1022,7 +1103,7 @@ lav_model_gradient_DD <- function(lavmodel, GLIST = NULL, group = 1L) {
         nr <- nc <- nrow(MLIST$beta)
         lv.idx <- 1:nfac
         # MUST BE ROWWISE!
-        beta.idx <- rep(nr*((1:nfac) - 1L), times=nfac) + rep(lv.idx, each=nfac) 
+        beta.idx <- rep(nr*((1:nfac) - 1L), times=nfac) + rep(lv.idx, each=nfac)
         DD$beta <- Delta.beta[beta.idx,,drop=FALSE]
     }
 
