@@ -16,7 +16,9 @@ lav_model_nvcov_bootstrap <- function(lavmodel       = NULL,
     }
 
     boot.type <- "ordinary"
-    if(lavoptions$test == "bollen.stine") boot.type <- "bollen.stine"
+    if("bollen.stine" %in% lavoptions$test) {
+        boot.type <- "bollen.stine"
+    }
 
     TEST <- NULL
     COEF <- bootstrap.internal(object          = NULL,
@@ -370,10 +372,33 @@ lav_model_vcov <- function(lavmodel       = NULL,
     mimic       <- lavoptions$mimic
 
     # special cases
-    if(se == "none" || se == "external") {
+    if(se == "none" || se == "external" || se == "twostep") {
         return( matrix(0, 0, 0) )
     }
 
+    # bordered efa rotation?
+    if( (.hasSlot(lavmodel, "nefa")) && (lavmodel@nefa > 0L) &&
+        (lavoptions$rotation != "none") &&
+        (lavoptions$rotation.se == "bordered") ) {
+
+        con.idx <- which(lavpartable$op %in% c("==", "<", ">"))
+        efa.idx <- which(lavpartable$user[con.idx] == 7L)
+        con.jac <- lavmodel@con.jac
+        inactive.idx <- attr(con.jac, "inactive.idx")
+        ceq.idx <- attr(con.jac, "ceq.idx")
+        if(length(efa.idx) > 0L) {
+             con.jac <- con.jac[-efa.idx, , drop = FALSE]
+        }
+        ceq.efa.JAC <- lavmodel@ceq.efa.JAC
+        # only take first efa.idx rows
+        ceq.efa.JAC <- ceq.efa.JAC[ seq_len(length(efa.idx)), ,
+                                    drop = FALSE ]
+        lavmodel@con.jac <- rbind(con.jac, ceq.efa.JAC)
+        attr(lavmodel@con.jac, "inactive.idx") <- inactive.idx
+        attr(lavmodel@con.jac, "ceq.idx") <- ceq.idx
+        attr(lavmodel@con.jac, "efa.idx") <- efa.idx
+    }
+
     if(se == "standard") {
         NVarCov <- lav_model_information(lavmodel       = lavmodel,
                                          lavsamplestats = lavsamplestats,
