@@ -53,7 +53,11 @@ lav_options_default <- function(mimic = "lavaan") {
                 conditional.x      = "default", # or FALSE?
                 fixed.x            = "default", # or FALSE?
                 orthogonal         = FALSE,
+                orthogonal.x       = FALSE,
+                orthogonal.y       = FALSE,
                 std.lv             = FALSE,
+                effect.coding      = FALSE,     # TRUE implies
+                                                # c("loadings", "intercepts")
                 parameterization   = "default",
 
                 auto.fix.first     = FALSE,
@@ -63,10 +67,39 @@ lav_options_default <- function(mimic = "lavaan") {
                 auto.cov.y         = FALSE,
                 auto.th            = FALSE,
                 auto.delta         = FALSE,
+                auto.efa           = FALSE,
+
+                # seat belts
+                safe.ov.var.ub     = FALSE,
+                save.ov.var.lb     = FALSE,
+
+                # rotation
+                rotation           = "geomin",
+                rotation.se        = "delta", # "bordered" or "delta"
+                rotation.args      = list(orthogonal     = FALSE,
+                                          row.weights    = "none",
+                                          std.ov         = FALSE,
+                                          geomin.epsilon = 0.01,
+                                          orthomax.gamma = 1,
+                                          cf.gamma       = 0,
+                                          oblimin.gamma  = 0,
+                                          target         = matrix(0,0,0),
+                                          target.mask    = matrix(0,0,0),
+                                          rstarts        = 100L,
+                                          algorithm      = "gpa",
+                                          reflect        = TRUE,
+                                          order.lv.by    = "index",
+                                          gpa.tol        = 1e-05,
+                                          tol            = 1e-08,
+                                          warn           = FALSE,
+                                          verbose        = FALSE,
+                                          jac.init.rot   = TRUE,
+                                          max.iter       = 10000L),
 
                 # full data
                 std.ov             = FALSE,
                 missing            = "default",
+                sampling.weights.normalization = "total",
 
                 # summary data
                 sample.cov.rescale = "default",
@@ -112,6 +145,7 @@ lav_options_default <- function(mimic = "lavaan") {
                 optim.init_nelder_mead = FALSE,
                 optim.var.transform    = "none",
                 optim.parscale         = "none",
+                optim.dx.tol           = 1e-03, # not too scrict
                 em.iter.max            = 10000L,
                 em.fx.tol              = 1e-08,
                 em.dx.tol              = 1e-04,
@@ -174,10 +208,8 @@ lav_options_set <- function(opt = NULL) {
     opt.old <- opt
     opt <- lapply(opt, function(x) { if(is.character(x)) tolower(x) else x})
     # except group,group.partial, which may contain capital letters
-    opt$group <- opt.old$group
     opt$group.label <- opt.old$group.label
     opt$group.partial <- opt.old$group.partial
-    opt$cluster <- opt.old$cluster
 
 
     # do.fit implies se="none and test="none" (unless not default)
@@ -185,7 +217,7 @@ lav_options_set <- function(opt = NULL) {
         if(opt$se == "default") {
             opt$se <- "none"
         }
-        if(opt$test == "default") {
+        if(length(opt$test) == 1L && opt$test == "default") {
             opt$test <- "none"
         }
     }
@@ -207,12 +239,16 @@ lav_options_set <- function(opt = NULL) {
     # group.equal and group.partial
     if(opt$group.equal[1] == "none") {
         opt$group.equal <- character(0)
-    } else if(is.null(opt$group.equal) || nchar(opt$group.equal) == 0L) {
-        if(opt$mimic == "Mplus" && !is.null(opt$group)) {
+    } else if(is.null(opt$group.equal) || all(nchar(opt$group.equal) == 0L)) {
+        if(opt$mimic == "Mplus") {
             if(opt$categorical) {
                 opt$group.equal <- c("loadings", "thresholds")
             } else {
-                opt$group.equal <- c("loadings", "intercepts")
+                if(is.logical(opt$meanstructure) && !opt$meanstructure) {
+                    opt$group.equal <- "loadings"
+                } else {
+                    opt$group.equal <- c("loadings", "intercepts")
+                }
             }
         } else {
             opt$group.equal <- character(0)
@@ -220,15 +256,21 @@ lav_options_set <- function(opt = NULL) {
     } else if(length(opt$group.equal) == 0) {
         # nothing to do
     } else if(all(opt$group.equal %in% c("loadings", "intercepts", "means",
+                                         "composite.loadings",
                                          "regressions", "residuals",
                                          "residual.covariances", "thresholds",
                                          "lv.variances", "lv.covariances"))) {
         # nothing to do
     } else {
+        wrong.idx <- which(!opt$group.equal %in%
+            c("loadings", "intercepts", "means",
+              "composite.loadings", "regressions", "residuals",
+              "residual.covariances", "thresholds",
+              "lv.variances", "lv.covariances"))
         stop("lavaan ERROR: unknown value for `group.equal' argument: ",
-             opt$group.equal, "\n")
+             sQuote(opt$group.equal[wrong.idx[1L]]), "\n")
     }
-    if(is.null(opt$group.partial) || nchar(opt$group.partial) == 0L) {
+    if(is.null(opt$group.partial) || all(nchar(opt$group.partial) == 0L)) {
         opt$group.partial <- character(0)
     } else if(length(opt$group.partial) == 0) {
         # nothing to do
@@ -276,11 +318,11 @@ lav_options_set <- function(opt = NULL) {
         }
 
         # test
-        if(opt$test == "default") {
+        if(length(opt$test) == 1L && opt$test == "default") {
             opt$test <- "yuan.bentler.mplus"
-        } else if(opt$test %in% c("none", "standard",
+        } else if(all(opt$test %in% c("none", "standard",
                                   "satorra.bentler",
-                                  "yuan.bentler","yuan.bentler.mplus")) {
+                                  "yuan.bentler","yuan.bentler.mplus"))) {
             # nothing to do
         } else if(opt$se == "robust") {
             opt$test <- "yuan.bentler.mplus"
@@ -325,9 +367,9 @@ lav_options_set <- function(opt = NULL) {
         opt$missing <- "listwise"
 
         # test
-        if(opt$test == "default") {
+        if(length(opt$test) == 1L && opt$test == "default") {
             opt$test <- "standard"
-        } else if(opt$test %in% c("none", "standard","yuan.bentler")) {
+        } else if(all(opt$test %in% c("none", "standard","yuan.bentler"))) {
             # nothing to do
         } else if(opt$se == "robust") {
             opt$test <- "yuan.bentler"
@@ -439,58 +481,43 @@ lav_options_set <- function(opt = NULL) {
     }
 
     # default test statistic
-    if(opt$test == "default") {
+    if(length(opt$test) == 1L && opt$test == "default") {
         if(opt$missing == "two.stage" ||
            opt$missing == "robust.two.stage") {
             opt$test <- "satorra.bentler"
         } else {
             opt$test <- "standard"
         }
-    } else if(opt$test %in% c("none", "standard")) {
-        # nothing to do
-    } else if(opt$test == "satorra" ||
-              opt$test == "sb" ||
-              opt$test == "SB" ||
-              opt$test == "satorra.bentler" ||
-              opt$test == "satorra-bentler") {
-        opt$test <- "satorra.bentler"
-    } else if(opt$test == "yuan" ||
-              opt$test == "yb" ||
-              opt$test == "YB" ||
-              opt$test == "yuan.bentler" ||
-              opt$test == "yuan-bentler") {
-        opt$test <- "yuan.bentler"
-    } else if(opt$test == "yuan.bentler.mplus" ||
-              opt$test == "yuan-bentler.mplus" ||
-              opt$test == "yuan-bentler-mplus") {
-        opt$test <- "yuan.bentler.mplus"
-    } else if(opt$test == "m.adjusted" ||
-              opt$test == "m" ||
-              opt$test == "mean.adjusted" ||
-              opt$test == "mean-adjusted") {
-        opt$test <- "satorra.bentler"
-    } else if(opt$test == "mean.var.adjusted" ||
-              opt$test == "mean-var-adjusted" ||
-              opt$test == "mv" ||
-              opt$test == "second.order" ||
-              opt$test == "satterthwaite" ||
-              opt$test == "Satterthwaite" ||
-              opt$test == "mv.adjusted") {
-        opt$test <- "mean.var.adjusted"
-    } else if(opt$test == "mplus6" ||
-              opt$test == "scale.shift" ||
-              opt$test == "scaled.shifted") {
-        opt$test <- "scaled.shifted"
-    } else if(opt$test == "bootstrap" ||
-              opt$test == "boot" ||
-              opt$test == "bollen.stine" ||
-              opt$test == "bollen-stine") {
-        opt$test <- "bollen.stine"
-    } else {
-        stop("lavaan ERROR: `test' argument must one of \"none\", \"standard\",
-            \"satorra.bentler\", \"yuan.bentler\", \"yuan.bentler.mplus\",
-            \"mean.var.adjusted\", \"scaled.shifted\",
-            \"bollen.stine\", or \"bootstrap\"")
+    }
+
+    # rename if needed
+    if(length(target.idx <- which(opt$test %in%
+        c("satorra", "sb", "satorra.bentler", "satorra-bentler",
+          "m.adjusted", "m", "mean.adjusted", "mean-adjusted"))) > 0L) {
+        opt$test[target.idx] <- "satorra.bentler"
+    }
+    if(length(target.idx <- which(opt$test %in%
+        c("yuan", "yb", "yuan.bentler", "yuan-bentler"))) > 0L) {
+        opt$test[target.idx] <- "yuan.bentler"
+    }
+    if(length(target.idx <- which(opt$test %in%
+        c("yuan.bentler.mplus", "yuan-bentler.mplus",
+          "yuan-bentler-mplus"))) > 0L) {
+        opt$test[target.idx] <- "yuan.bentler.mplus"
+    }
+    if(length(target.idx <- which(opt$test %in%
+        c("mean.var.adjusted", "mean-var-adjusted", "mv", "second.order",
+          "satterthwaite", "mv.adjusted"))) > 0L) {
+        opt$test[target.idx] <- "mean.var.adjusted"
+    }
+    if(length(target.idx <- which(opt$test %in%
+        c("mplus6", "scale.shift", "scaled.shifted",
+          "scaled-shifted"))) > 0L) {
+        opt$test[target.idx] <- "scaled.shifted"
+    }
+    if(length(target.idx <- which(opt$test %in%
+        c("bootstrap", "boot", "bollen.stine", "bollen-stine"))) > 0L) {
+        opt$test[target.idx] <- "bollen.stine"
     }
 
     # check missing
@@ -501,11 +528,10 @@ lav_options_set <- function(opt = NULL) {
         opt$missing <- "listwise"
     }
     if(opt$missing %in% c("ml", "ml.x") &&
-       opt$test %in% c("satorra.bentler",
-                       "mean.var.adjusted", "scaled.shifted")) {
+       any(opt$test %in% c("satorra.bentler",
+                       "mean.var.adjusted", "scaled.shifted"))) {
         warning("lavaan WARNING: missing will be set to ",
-                    dQuote("listwise"), " for test = ",
-                    dQuote(opt$test) )
+                    dQuote("listwise"), " for satorra.bentler style test")
         opt$missing <- "listwise"
     }
 
@@ -544,9 +570,18 @@ lav_options_set <- function(opt = NULL) {
         }
         # observed.information (ALWAYS "h1" for now)
         opt$observed.information <- "h1"
+
+
         # test
-        if(opt$test == "default" ||
-           opt$test == "satorra.bentler") {
+        if(length(opt$test) > 1L) {
+            warning("lavaan WARNING: test= argument can only contain a single element if missing = ", dQuote(opt$missing), " (taking the first)" )
+            opt$test <- opt$test[1]
+        }
+
+        if(length(opt$test) == 1L && opt$test == "default") {
+            opt$test <- "satorra.bentler"
+        } else if(length(opt$test) == 1L && opt$test %in%
+            c("satorra", "sb", "satorra.bentler", "satorra-bentler")) {
             opt$test <- "satorra.bentler"
         } else {
             warning("lavaan WARNING: test will be set to ",
@@ -574,11 +609,11 @@ lav_options_set <- function(opt = NULL) {
         # by default: no meanstructure!
         if(opt$estimator == "pml") {
             opt$meanstructure <- TRUE
+        } else if(opt$mimic == "Mplus") {
+            opt$meanstructure <- TRUE
         } else {
             opt$meanstructure <- FALSE
         }
-        # unless there is a group argument? (added since 0.4-10)
-        # if(!is.null(opt$group)) opt$meanstructure <- TRUE
     } else {
         stop("lavaan ERROR: meanstructure must be TRUE, FALSE or \"default\"\n")
     }
@@ -640,7 +675,7 @@ lav_options_set <- function(opt = NULL) {
               opt$estimator == "mlmv"  ||
               opt$estimator == "mlmvs") {
         est.orig <- opt$estimator
-        if(opt$test != "none") {
+        if(! (length(opt$test) == 1L && opt$test == "none") ) {
             if(opt$estimator == "mlm") {
                 opt$test <- "satorra.bentler"
             } else if(opt$estimator == "mlmv") {
@@ -679,9 +714,12 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: use ML estimator for bootstrap")
         }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.huber.white"
-        if(opt$test != "none" && opt$se != "external") {
-            if(opt$mimic == "Mplus" || opt$test == "yuan.bentler.mplus") {
+        if( !(length(opt$test) == 1L && opt$test == "none") &&
+            opt$se != "external" ) {
+            if(opt$mimic == "Mplus") {
                 opt$test <- "yuan.bentler.mplus"
+            } else if(opt$mimic == "EQS") {
+                opt$test <- "yuan.bentler"
             } else {
                 opt$test <- "yuan.bentler.mplus" # for now
             }
@@ -699,7 +737,7 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: invalid value for `se' argument when estimator is GLS: ",
                  opt$se, "\n")
         }
-        if(!opt$test %in% c("standard","none")) {
+        if(!all(opt$test %in% c("standard","none"))) {
             stop("lavaan ERROR: invalid value for `test' argument when estimator is GLS: ",
                  opt$test, "\n")
         }
@@ -716,7 +754,7 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: invalid value for `se' argument when estimator is NTRLS: ",
                  opt$se, "\n")
         }
-        if(!opt$test %in% c("standard","none")) {
+        if(!all(opt$test %in% c("standard","none"))) {
             stop("lavaan ERROR: invalid value for `test' argument when estimator is NTRLS: ",
                  opt$test, "\n")
         }
@@ -737,7 +775,7 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: invalid value for `se' argument when estimator is WLS: ",
                  opt$se, "\n")
         }
-        if(!opt$test %in% c("standard","none")) {
+        if(!all(opt$test %in% c("standard","none"))) {
             stop("lavaan ERROR: invalid value for `test' argument when estimator is WLS: ",
                  opt$test, "\n")
         }
@@ -758,9 +796,9 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: invalid value for `se' argument when estimator is DWLS: ",
                  opt$se, "\n")
         }
-        if(!opt$test %in% c("standard","none","satorra.bentler",
+        if(!all(opt$test %in% c("standard","none","satorra.bentler",
                             "mean.adjusted",
-                            "mean.var.adjusted","scaled.shifted")) {
+                            "mean.var.adjusted","scaled.shifted"))) {
             stop("lavaan ERROR: invalid value for `test' argument when estimator is DWLS: ",
                  opt$test, "\n")
         }
@@ -771,10 +809,10 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: use (D)WLS estimator for bootstrap")
         }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
-                           "scaled.shifted")) {
+        if(all(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted"))) {
             # nothing to do
-        } else if(opt$test != "none") {
+        } else if(! (length(opt$test) == 1L && opt$test == "none") ) {
             opt$test <- "satorra.bentler"
         }
         #opt$missing <- "listwise"
@@ -784,10 +822,10 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: use (D)WLS estimator for bootstrap")
         }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
-                           "scaled.shifted")) {
+        if(all(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted"))) {
             # nothing to do
-        } else if(opt$test != "none") {
+        } else if(! (length(opt$test) == 1L && opt$test == "none") ) {
             opt$test <- "scaled.shifted"
         }
         #opt$missing <- "listwise"
@@ -797,10 +835,10 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: use (D)WLS estimator for bootstrap")
         }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
-                           "scaled.shifted")) {
+        if(all(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted"))) {
             # nothing to do
-        } else if(opt$test != "none") {
+        } else if(! (length(opt$test) == 1L && opt$test == "none") ) {
             opt$test <- "mean.var.adjusted"
         }
         #opt$missing <- "listwise"
@@ -820,9 +858,9 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: invalid value for `se' argument when estimator is ULS: ",
                  opt$se, "\n")
         }
-        if(!opt$test %in% c("standard","none", "satorra.bentler",
+        if(!all(opt$test %in% c("standard","none", "satorra.bentler",
                             "mean.adjusted",
-                            "mean.var.adjusted","scaled.shifted")) {
+                            "mean.var.adjusted","scaled.shifted"))) {
             stop("lavaan ERROR: invalid value for `test' argument when estimator is ULS: ",
                  opt$test, "\n")
         }
@@ -833,10 +871,10 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: use ULS estimator for bootstrap")
         }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
-                           "scaled.shifted")) {
+        if(all(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted"))) {
             # nothing to do
-        } else if(opt$test != "none") {
+        } else if(! (length(opt$test) == 1L && opt$test == "none") ) {
             opt$test <- "satorra.bentler"
         }
         #opt$missing <- "listwise"
@@ -846,10 +884,10 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: use ULS estimator for bootstrap")
         }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
-                           "scaled.shifted")) {
+        if(all(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted"))) {
             # nothing to do
-        } else if(opt$test != "none") {
+        } else if(! (length(opt$test) == 1L && opt$test == "none") ) {
             opt$test <- "scaled.shifted"
         }
         #opt$missing <- "listwise"
@@ -859,41 +897,52 @@ lav_options_set <- function(opt = NULL) {
             stop("lavaan ERROR: use ULS estimator for bootstrap")
         }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
-                           "scaled.shifted")) {
+        if(all(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted"))) {
             # nothing to do
-        } else if(opt$test != "none") {
+        } else if(! (length(opt$test) == 1L && opt$test == "none") ) {
             opt$test <- "mean.var.adjusted"
         }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "pml") {
         opt$estimator <- "PML"
         opt$information <- "observed"
-        if(opt$se == "default")
+        if(opt$se == "default") {
             opt$se <- "robust.huber.white"
-        if(opt$test != "none") opt$test <- "mean.var.adjusted"
+        }
+        if(! (length(opt$test) == 1L && opt$test == "none") ) {
+            opt$test <- "mean.var.adjusted"
+        }
         #opt$missing <- "listwise"
     } else if(opt$estimator %in% c("fml","umn")) {
         opt$estimator <- "FML"
         opt$information <- "observed"
-        if(opt$se == "default")
+        if(opt$se == "default") {
             opt$se <- "standard"
-        if(opt$test != "none") opt$test <- "standard"
+        }
+        if(! (length(opt$test) == 1L && opt$test == "none") ) {
+            opt$test <- "standard"
+        }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "reml") {
         opt$estimator <- "REML"
         opt$information <- "observed"
-        if(opt$se == "default")
+        if(opt$se == "default") {
             opt$se <- "standard"
-        if(opt$test != "none") opt$test <- "standard"
+        }
+        if(! (length(opt$test) == 1L && opt$test == "none") ) {
+            opt$test <- "standard"
+        }
         opt$missing <- "listwise"
     } else if(opt$estimator %in% c("mml")) {
         opt$estimator <- "MML"
         opt$information <- "observed"
-        if(opt$se == "default")
+        if(opt$se == "default") {
             opt$se <- "standard"
-        if(opt$test == "default")
+        }
+        if(length(opt$test) == 1L && opt$test == "default") {
             opt$test <- "none"
+        }
         #opt$missing <- "listwise"
         if(opt$link == "default") {
             #opt$link <- "logit"
@@ -913,7 +962,7 @@ lav_options_set <- function(opt = NULL) {
         if(opt$se == "default") {
             opt$se <- "none"
         }
-        if(opt$test == "default") {
+        if(length(opt$test) == 1L && opt$test == "default") {
             opt$test <- "none"
         }
     } else {
@@ -1121,6 +1170,73 @@ lav_options_set <- function(opt = NULL) {
         stop("lavaan ERROR: argument `parameterization' should be `delta' or `theta'")
     }
 
+
+
+    # std.lv vs auto.fix.first # new in 0.6-5 (used to be in sem/cfa/growth)
+    if(opt$std.lv) {
+        opt$auto.fix.first <- FALSE
+    }
+
+    # std.lv vs effect.coding # new in 0.6-4
+    if(is.logical(opt$effect.coding)) {
+        if(opt$effect.coding) {
+            opt$effect.coding <- c("loadings", "intercepts")
+        } else {
+            opt$effect.coding <- ""
+        }
+    } else if(length(opt$effect.coding) == 0L) {
+        # nothing to do
+    } else if(all(opt$effect.coding %in% c("loadings", "intercepts",
+                                           "mg.lv.efa.variances",
+                                           "mg.lv.variances",
+                                           "mg.lv.means",
+                                           "mg.lv.intercepts"))) {
+        # nothing to do
+    } else {
+        stop("lavaan ERROR: unknown value for ", sQuote("effect.coding"),
+             " argument: ", opt$effect.coding, "\n")
+    }
+
+    # if we use effect coding for the factor loadings, we don't need/want
+    # std.lv = TRUE
+    if("loadings" %in% opt$effect.coding) {
+        if(opt$std.lv) {
+            stop("lavaan ERROR: std.lv is set to FALSE but effect.coding contains ", dQuote("loadings"))
+            opt$std.lv <- FALSE
+        }
+        # shut off auto.fix.first
+        opt$auto.fix.first <- FALSE
+    }
+
+    # test again
+
+    # unless test = "none", always add test = "standard" as the
+    # first entry
+    # NO: this breaks lavaan.survey pval.pFsum, which has the following check:
+    # if (!lavInspect(lavaan.fit, "options")$test %in% c("satorra.bentler",
+    #    "mean.var.adjusted", "Satterthwaite")) {
+    #    stop("Please refit the model with Satorra-Bentler (MLM) or Satterthwaite (MLMVS) adjustment.")
+    #}
+    #if(! (length(opt$test) == 1L && opt$test == "none") ) {
+    #    opt$test <- c("standard", opt$test)
+    #    opt$test <- unique(opt$test)
+    #}
+
+    # final check
+    wrong.idx <- which(! opt$test %in% c("none", "standard", "satorra.bentler",
+                            "yuan.bentler", "yuan.bentler.mplus",
+                            "mean.var.adjusted", "scaled.shifted",
+                            "bollen.stine"))
+    if(length(wrong.idx) > 0L) {
+        stop("lavaan ERROR: invalid option(s) for test argument: ",
+             paste(dQuote(opt$test[wrong.idx]), collapse = " "),
+             "\n\t\t",
+             "Possible options are: \"none\", \"standard\",
+             \"satorra.bentler\", \"yuan.bentler\", \"yuan.bentler.mplus\",
+             \"mean.var.adjusted\", \"scaled.shifted\", or \"bollen.stine\"")
+    }
+
+
     if(opt$debug) { cat("lavaan DEBUG: lavaanOptions OUT\n"); str(opt) }
 
     opt
