@@ -11,7 +11,8 @@
 #
 lavTestScore <- function(object, add = NULL, release = NULL,
                          univariate = TRUE, cumulative = FALSE,
-                         epc = FALSE, verbose = FALSE, warn = TRUE,
+                         epc = FALSE, standardized = epc, cov.std = epc,
+                         verbose = FALSE, warn = TRUE,
                          information = "expected") {
 
     # check object
@@ -35,7 +36,7 @@ lavTestScore <- function(object, add = NULL, release = NULL,
 
 
     # Mode 1: ADDING new parameters
-    if(!is.null(add) && nchar(add) > 0L) {
+    if(!is.null(add) && all(nchar(add) > 0L)) {
         # check release argument
         if(!is.null(release)) {
             stop("lavaan ERROR: `add' and `release' arguments cannot be used together.")
@@ -169,6 +170,7 @@ lavTestScore <- function(object, add = NULL, release = NULL,
 
     if(univariate) {
         TS <- numeric( nrow(R) )
+        EPC.uni <- numeric( nrow(R) ) # ignored in release= mode
         for(r in r.idx) {
             R1 <- R[-r,,drop = FALSE]
             Z1 <- cbind( rbind(Information, R1),
@@ -176,18 +178,30 @@ lavTestScore <- function(object, add = NULL, release = NULL,
             Z1.plus <- MASS::ginv(Z1)
             Z1.plus1 <- Z1.plus[ 1:nrow(Information), 1:nrow(Information) ]
             TS[r] <- as.numeric(N * t(score) %*%  Z1.plus1 %*% score)
+            if (epc && !is.null(add)) {
+              #EPC.uni[r] <- -1 * utils::tail(as.numeric(score %*%  Z1.plus1),
+              #                               n = nrow(R))[r]
+              # to keep the 'sign' consistent with modindices(), which
+              # uses epc = 'new - old'
+              EPC.uni[r] <- +1 * utils::tail(as.numeric(score %*%  Z1.plus1),
+                                             n = nrow(R))[r]
+            }
         }
 
         Table2 <- Table
         Table2$X2 <- TS[r.idx]
         Table2$df <- rep(1, length(r.idx))
         Table2$p.value <- 1 - pchisq(Table2$X2, df = Table2$df)
+        if (epc && !is.null(add)) {
+          Table2$epc <- EPC.uni[r.idx]
+        }
         attr(Table2, "header") <- "univariate score tests:"
         OUT$uni <- Table2
     }
 
     if(cumulative) {
         TS.order <- sort.int(TS, index.return = TRUE, decreasing = TRUE)$ix
+        ROW.order <- sort.int(TS[r.idx], index.return = TRUE, decreasing = TRUE)$ix
         TS <- numeric( length(r.idx) )
         for(r in 1:length(r.idx)) {
             rcumul.idx <- TS.order[1:r]
@@ -200,7 +214,7 @@ lavTestScore <- function(object, add = NULL, release = NULL,
             TS[r] <- as.numeric(N * t(score) %*%  Z1.plus1 %*% score)
         }
 
-        Table3 <- Table
+        Table3 <- Table[ROW.order, ]
         Table3$X2 <- TS
         Table3$df <- seq_len( length(TS) )
         Table3$p.value <- 1 - pchisq(Table3$X2, df = Table3$df)
@@ -228,16 +242,17 @@ lavTestScore <- function(object, add = NULL, release = NULL,
                      rbind(t(R1), matrix(0,nrow(R1),nrow(R1))) )
         Z1.plus <- MASS::ginv(Z1)
         Z1.plus1 <- Z1.plus[ 1:nrow(Information), 1:nrow(Information) ]
-        EPC.all <- -1 * as.numeric(score %*%  Z1.plus1)
+        #EPC.all <- -1 * as.numeric(score %*%  Z1.plus1)
+        # to keep the 'sign' consistent with modindices(), which
+        # uses epc = 'new - old'
+        EPC.all <- +1 * as.numeric(score %*%  Z1.plus1)
 
         # create epc table for the 'free' parameters
-        if (!is.null(add) && nchar(add) > 0L) {
-          LIST <- parTable(FIT)[,c("lhs","op","rhs","group",
-                                   "user","free","label","plabel")]
+        if (!is.null(add) && all(nchar(add) > 0L)) {
+          LIST <- parTable(FIT)
         } else {
           ## release mode
-          LIST <- parTable(object)[,c("lhs","op","rhs","group",
-                                      "user","free","label","plabel")]
+          LIST <- parTable(object)
         }
         if(lav_partable_ngroups(LIST) == 1L) {
             LIST$group <- NULL
@@ -252,8 +267,66 @@ lavTestScore <- function(object, add = NULL, release = NULL,
         LIST$epc <- rep(as.numeric(NA), length(LIST$lhs))
         LIST$epc[ LIST$free > 0 ] <- EPC.all
         LIST$epv <- LIST$est + LIST$epc
+
+        if (standardized) {
+
+          EPC <- LIST$epc
+
+          if (cov.std) {
+            # replace epc values for variances by est values
+            var.idx <- which(LIST$op == "~~" & LIST$lhs == LIST$rhs &
+                               LIST$exo == 0L)
+            EPC[ var.idx ] <- LIST$est[ var.idx ]
+          }
+
+          # two problems:
+          #   - EPC of variances can be negative, and that is
+          #     perfectly legal
+          #   - EPC (of variances) can be tiny (near-zero), and we should
+          #     not divide by tiny variables
+          small.idx <- which(LIST$op == "~~" &
+                               LIST$lhs == LIST$rhs &
+                               abs(EPC) < sqrt( .Machine$double.eps ) )
+          if (length(small.idx) > 0L) {
+            EPC[ small.idx ] <- as.numeric(NA)
+          }
+
+          # get the sign
+          EPC.sign <- sign(LIST$epc)
+
+          LIST$sepc.lv <- EPC.sign * lav_standardize_lv(object,
+                                                        partable = LIST,
+                                                        est = abs(EPC),
+                                                        cov.std = cov.std)
+          if (length(small.idx) > 0L) {
+            LIST$sepc.lv[small.idx] <- 0
+          }
+          LIST$sepc.all <- EPC.sign * lav_standardize_all(object,
+                                                          partable = LIST,
+                                                          est = abs(EPC),
+                                                          cov.std = cov.std)
+          if (length(small.idx) > 0L) {
+            LIST$sepc.all[small.idx] <- 0
+          }
+          LIST$sepc.nox <- EPC.sign * lav_standardize_all_nox(object,
+                                                              partable = LIST,
+                                                              est = abs(EPC),
+                                                              cov.std = cov.std)
+          if (length(small.idx) > 0L) {
+            LIST$sepc.nox[small.idx] <- 0
+          }
+
+        }
+
         LIST$free[ LIST$user == 10L ] <- 0
         LIST$user <- NULL
+        # remove some more columns
+        LIST$id <- LIST$ustart <- LIST$exo <- LIST$start <- LIST$se <- LIST$prior <- NULL
+        if(lav_partable_nblocks(LIST) == 1L) {
+          LIST$block <- NULL
+          LIST$group <- NULL
+          LIST$level <- NULL
+        }
 
         attr(LIST, "header") <- "expected parameter changes (epc) and expected parameter values (epv):"
 
